from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.uris import Uris


T = TypeVar("T", bound="ItemLogin")


@_attrs_define
class ItemLogin:
    """
    Attributes:
        password (Union[Unset, str]):
        totp (Union[Unset, str]):
        uris (Union[Unset, Uris]):
        username (Union[Unset, str]):
    """

    password: Union[Unset, str] = UNSET
    totp: Union[Unset, str] = UNSET
    uris: Union[Unset, "Uris"] = UNSET
    username: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        password = self.password

        totp = self.totp

        uris: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.uris, Unset):
            uris = self.uris.to_dict()

        username = self.username

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if password is not UNSET:
            field_dict["password"] = password
        if totp is not UNSET:
            field_dict["totp"] = totp
        if uris is not UNSET:
            field_dict["uris"] = uris
        if username is not UNSET:
            field_dict["username"] = username

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.uris import Uris

        d = src_dict.copy()
        password = d.pop("password", UNSET)

        totp = d.pop("totp", UNSET)

        _uris = d.pop("uris", UNSET)
        uris: Union[Unset, Uris]
        if isinstance(_uris, Unset):
            uris = UNSET
        else:
            uris = Uris.from_dict(_uris)

        username = d.pop("username", UNSET)

        item_login = cls(
            password=password,
            totp=totp,
            uris=uris,
            username=username,
        )

        item_login.additional_properties = d
        return item_login

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.item_template_reprompt import ItemTemplateReprompt
from ..models.item_template_type import ItemTemplateType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.field import Field
    from ..models.item_card import ItemCard
    from ..models.item_identity import ItemIdentity
    from ..models.item_login import ItemLogin
    from ..models.item_secure_note import ItemSecureNote


T = TypeVar("T", bound="ItemTemplate")


@_attrs_define
class ItemTemplate:
    """
    Attributes:
        card (Union[Unset, ItemCard]):
        collection_ids (Union[Unset, list[UUID]]):
        favorite (Union[Unset, bool]):
        fields (Union[Unset, list['Field']]):
        folder_id (Union[Unset, UUID]):
        identity (Union[Unset, ItemIdentity]):
        login (Union[Unset, ItemLogin]):
        name (Union[Unset, str]):
        notes (Union[Unset, str]):
        organization_id (Union[Unset, UUID]):
        reprompt (Union[Unset, ItemTemplateReprompt]):
        secure_note (Union[Unset, ItemSecureNote]):
        type_ (Union[Unset, ItemTemplateType]):
    """

    card: Union[Unset, "ItemCard"] = UNSET
    collection_ids: Union[Unset, list[UUID]] = UNSET
    favorite: Union[Unset, bool] = UNSET
    fields: Union[Unset, list["Field"]] = UNSET
    folder_id: Union[Unset, UUID] = UNSET
    identity: Union[Unset, "ItemIdentity"] = UNSET
    login: Union[Unset, "ItemLogin"] = UNSET
    name: Union[Unset, str] = UNSET
    notes: Union[Unset, str] = UNSET
    organization_id: Union[Unset, UUID] = UNSET
    reprompt: Union[Unset, ItemTemplateReprompt] = UNSET
    secure_note: Union[Unset, "ItemSecureNote"] = UNSET
    type_: Union[Unset, ItemTemplateType] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        card: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.card, Unset):
            card = self.card.to_dict()

        collection_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.collection_ids, Unset):
            collection_ids = []
            for collection_ids_item_data in self.collection_ids:
                collection_ids_item = str(collection_ids_item_data)
                collection_ids.append(collection_ids_item)

        favorite = self.favorite

        fields: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = []
            for fields_item_data in self.fields:
                fields_item = fields_item_data.to_dict()
                fields.append(fields_item)

        folder_id: Union[Unset, str] = UNSET
        if not isinstance(self.folder_id, Unset):
            folder_id = str(self.folder_id)

        identity: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.identity, Unset):
            identity = self.identity.to_dict()

        login: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.login, Unset):
            login = self.login.to_dict()

        name = self.name

        notes = self.notes

        organization_id: Union[Unset, str] = UNSET
        if not isinstance(self.organization_id, Unset):
            organization_id = str(self.organization_id)

        reprompt: Union[Unset, int] = UNSET
        if not isinstance(self.reprompt, Unset):
            reprompt = self.reprompt.value

        secure_note: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.secure_note, Unset):
            secure_note = self.secure_note.to_dict()

        type_: Union[Unset, int] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if card is not UNSET:
            field_dict["card"] = card
        if collection_ids is not UNSET:
            field_dict["collectionIds"] = collection_ids
        if favorite is not UNSET:
            field_dict["favorite"] = favorite
        if fields is not UNSET:
            field_dict["fields"] = fields
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if identity is not UNSET:
            field_dict["identity"] = identity
        if login is not UNSET:
            field_dict["login"] = login
        if name is not UNSET:
            field_dict["name"] = name
        if notes is not UNSET:
            field_dict["notes"] = notes
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if reprompt is not UNSET:
            field_dict["reprompt"] = reprompt
        if secure_note is not UNSET:
            field_dict["secureNote"] = secure_note
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.field import Field
        from ..models.item_card import ItemCard
        from ..models.item_identity import ItemIdentity
        from ..models.item_login import ItemLogin
        from ..models.item_secure_note import ItemSecureNote

        d = src_dict.copy()
        _card = d.pop("card", UNSET)
        card: Union[Unset, ItemCard]
        if isinstance(_card, Unset):
            card = UNSET
        else:
            card = ItemCard.from_dict(_card)

        collection_ids = []
        _collection_ids = d.pop("collectionIds", UNSET)
        for collection_ids_item_data in _collection_ids or []:
            collection_ids_item = UUID(collection_ids_item_data)

            collection_ids.append(collection_ids_item)

        favorite = d.pop("favorite", UNSET)

        fields = []
        _fields = d.pop("fields", UNSET)
        for fields_item_data in _fields or []:
            fields_item = Field.from_dict(fields_item_data)

            fields.append(fields_item)

        _folder_id = d.pop("folderId", UNSET)
        folder_id: Union[Unset, UUID]
        if isinstance(_folder_id, Unset):
            folder_id = UNSET
        else:
            folder_id = UUID(_folder_id)

        _identity = d.pop("identity", UNSET)
        identity: Union[Unset, ItemIdentity]
        if isinstance(_identity, Unset):
            identity = UNSET
        else:
            identity = ItemIdentity.from_dict(_identity)

        _login = d.pop("login", UNSET)
        login: Union[Unset, ItemLogin]
        if isinstance(_login, Unset):
            login = UNSET
        else:
            login = ItemLogin.from_dict(_login)

        name = d.pop("name", UNSET)

        notes = d.pop("notes", UNSET)

        _organization_id = d.pop("organizationId", UNSET)
        organization_id: Union[Unset, UUID]
        if isinstance(_organization_id, Unset):
            organization_id = UNSET
        else:
            organization_id = UUID(_organization_id)

        _reprompt = d.pop("reprompt", UNSET)
        reprompt: Union[Unset, ItemTemplateReprompt]
        if isinstance(_reprompt, Unset):
            reprompt = UNSET
        else:
            reprompt = ItemTemplateReprompt(_reprompt)

        _secure_note = d.pop("secureNote", UNSET)
        secure_note: Union[Unset, ItemSecureNote]
        if isinstance(_secure_note, Unset):
            secure_note = UNSET
        else:
            secure_note = ItemSecureNote.from_dict(_secure_note)

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, ItemTemplateType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ItemTemplateType(_type_)

        item_template = cls(
            card=card,
            collection_ids=collection_ids,
            favorite=favorite,
            fields=fields,
            folder_id=folder_id,
            identity=identity,
            login=login,
            name=name,
            notes=notes,
            organization_id=organization_id,
            reprompt=reprompt,
            secure_note=secure_note,
            type_=type_,
        )

        item_template.additional_properties = d
        return item_template

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.status_data_template_status import StatusDataTemplateStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="StatusDataTemplate")


@_attrs_define
class StatusDataTemplate:
    """
    Attributes:
        last_sync (Union[Unset, datetime.datetime]):
        server_url (Union[Unset, str]):
        status (Union[Unset, StatusDataTemplateStatus]):
        user_email (Union[Unset, str]):
        user_id (Union[Unset, UUID]):
    """

    last_sync: Union[Unset, datetime.datetime] = UNSET
    server_url: Union[Unset, str] = UNSET
    status: Union[Unset, StatusDataTemplateStatus] = UNSET
    user_email: Union[Unset, str] = UNSET
    user_id: Union[Unset, UUID] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        last_sync: Union[Unset, str] = UNSET
        if not isinstance(self.last_sync, Unset):
            last_sync = self.last_sync.isoformat()

        server_url = self.server_url

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        user_email = self.user_email

        user_id: Union[Unset, str] = UNSET
        if not isinstance(self.user_id, Unset):
            user_id = str(self.user_id)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if last_sync is not UNSET:
            field_dict["lastSync"] = last_sync
        if server_url is not UNSET:
            field_dict["serverUrl"] = server_url
        if status is not UNSET:
            field_dict["status"] = status
        if user_email is not UNSET:
            field_dict["userEmail"] = user_email
        if user_id is not UNSET:
            field_dict["userID"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        _last_sync = d.pop("lastSync", UNSET)
        last_sync: Union[Unset, datetime.datetime]
        if isinstance(_last_sync, Unset):
            last_sync = UNSET
        else:
            last_sync = isoparse(_last_sync)

        server_url = d.pop("serverUrl", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, StatusDataTemplateStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = StatusDataTemplateStatus(_status)

        user_email = d.pop("userEmail", UNSET)

        _user_id = d.pop("userID", UNSET)
        user_id: Union[Unset, UUID]
        if isinstance(_user_id, Unset):
            user_id = UNSET
        else:
            user_id = UUID(_user_id)

        status_data_template = cls(
            last_sync=last_sync,
            server_url=server_url,
            status=status,
            user_email=user_email,
            user_id=user_id,
        )

        status_data_template.additional_properties = d
        return status_data_template

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
import logging
import os
import shlex
import shutil
import subprocess
import time
import urllib


class BwProcess:
    def __init__(self, bw_cli_path, extra_env_vars=None):
        full_path = shutil.which(bw_cli_path)
        if not full_path:
            raise ValueError(f"{bw_cli_path} does not exists or is not executable")
        self.__bw = full_path
        self.__bw_process = None
        self.__bw_env_vars = {}
        if extra_env_vars:
            self.__bw_env_vars.update(extra_env_vars)

    def __construct_env(self, extra_env_vars=None):
        env = {}
        env.update(os.environ)
        env.update(self.__bw_env_vars)
        env.update(extra_env_vars or {})
        return env

    def run(self, *args, parse_output=False, timeout=30, extra_env_vars=None):
        logging_args = list(args)
        try:
            i = logging_args.index("login")
            for j in range(i + 1, len(logging_args)):
                logging_args[j] = "..."
        except ValueError:
            pass
        logging.info("Running %s", shlex.join(logging_args))
        args = [self.__bw, "--nointeraction", *list(args)]
        try:
            output = subprocess.check_output(args, timeout=timeout, env=self.__construct_env(extra_env_vars))
        except Exception as e:
            if isinstance(e, subprocess.CalledProcessError):
                logging.error("Failed to run %s. Exited with code %d", logging_args, e.returncode)
            else:
                logging.error("Failed to run %s. %s", logging_args, e)
            raise e
        if parse_output:
            return json.loads(output)

    def start_serve(
            self, password: str, host: str, port: int, *,
            username : str | None = None, clientid : str | None = None, clientsecret : str | None = None):
        if username:
            if clientid or clientsecret:
                raise ValueError("If username is set, clientid and/or clientsecret should not be set!")
        else:
            if (not clientid) or (not clientsecret):
                raise ValueError("If username is not set, both clientid and clientsecret must be set!")

        status_output = self.run("status", parse_output=True)
        assert status_output["status"] == "unauthenticated", f"Unexpected status: {status_output}"

        if username:
            self.run("login", username, "--passwordenv", "BW_PASSWORD", extra_env_vars={"BW_PASSWORD": password})
        else:
            self.run("login", "--apikey", extra_env_vars={"BW_CLIENTID": clientid, "BW_CLIENTSECRET": clientsecret})
            self.run("unlock", "--passwordenv", "BW_PASSWORD", extra_env_vars={"BW_PASSWORD": password})

        self.__bw_process = subprocess.Popen(
            [self.__bw, "serve", "--nointeraction", "--hostname", host, "--port", str(port)],
            bufsize=0,
            stderr=subprocess.STDOUT,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
            env=self.__construct_env(),
        )
        logging.info("Started bw serve with pid %d", self.__bw_process.pid)

        for _ in range(300):
            time.sleep(0.1)
            logging.info("Checking if bw serve is up")
            if self.__bw_process.poll() is not None:
                self.terminate_serve()
                outs, errs = self.__bw_process.communicate(timeout=15)
                souts, serrs = str(outs), str(errs)
                raise AssertionError(
                    f"bw serve (PID: {self.__bw_process.pid}) unexpectedly reports as terminated"
                    f"exit code: {self.__bw_process.returncode}, stdout: {souts}, stderr: {serrs}"
                )
            try:
                urllib.request.urlopen(f"http://{host}:{port}/", timeout=1)
            except urllib.error.HTTPError:
                # url responds with a 404
                logging.info("bw serve up")
                break
            except urllib.error.URLError:
                pass
        else:
            self.terminate_serve()
            outs, errs = self.__bw_process.communicate(timeout=15)
            souts, serrs = str(outs), str(errs)
            raise AssertionError(
                f"bw serve failed to start (PID: {self.__bw_process.pid}). "
                f"exit code: {self.__bw_process.returncode}, stdout: {souts}, stderr: {serrs}"
            )

    def terminate_serve(self):
        logging.info("Terminating bw serve")
        self.__bw_process.terminate()
        try:
            self.__bw_process.communicate(timeout=10)
            logging.info("bw serve (PID: %d) terminated", self.__bw_process.pid)
        except subprocess.TimeoutExpired:
            logging.warn("bw serve didn't terminate, killing it")
            self.__bw_process.kill()
        self.__bw_process = None
        self.run("logout")



from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Group")


@_attrs_define
class Group:
    """
    Attributes:
        hide_passwords (Union[Unset, bool]):
        id (Union[Unset, UUID]):
        read_only (Union[Unset, bool]):
    """

    hide_passwords: Union[Unset, bool] = UNSET
    id: Union[Unset, UUID] = UNSET
    read_only: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        hide_passwords = self.hide_passwords

        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        read_only = self.read_only

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if hide_passwords is not UNSET:
            field_dict["hidePasswords"] = hide_passwords
        if id is not UNSET:
            field_dict["id"] = id
        if read_only is not UNSET:
            field_dict["readOnly"] = read_only

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        hide_passwords = d.pop("hidePasswords", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = UUID(_id)

        read_only = d.pop("readOnly", UNSET)

        group = cls(
            hide_passwords=hide_passwords,
            id=id,
            read_only=read_only,
        )

        group.additional_properties = d
        return group

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.collection import Collection
from ...types import UNSET, Response


def _get_kwargs(
    id: UUID,
    *,
    body: Collection,
    organization_id: UUID,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    json_organization_id = str(organization_id)
    params["organizationId"] = json_organization_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": f"/object/org-collection/{id}",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 200:
        return None
    if response.status_code == 400:
        return None
    if response.status_code == 404:
        return None
    if response.status_code == 500:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: Collection,
    organization_id: UUID,
) -> Response[Any]:
    """Edit a Collection in a specified Organization.

     Edit an existing Collection in a specified Organization by specifying the unique collection
    identifier (e.g. `3a84be8d-12e7-4223-98cd-ae0000eabdec`) in the path, an Organization identifier as
    a query parameter, and Collection information in the request body.

    Args:
        id (UUID):
        organization_id (UUID):
        body (Collection):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
        organization_id=organization_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: Collection,
    organization_id: UUID,
) -> Response[Any]:
    """Edit a Collection in a specified Organization.

     Edit an existing Collection in a specified Organization by specifying the unique collection
    identifier (e.g. `3a84be8d-12e7-4223-98cd-ae0000eabdec`) in the path, an Organization identifier as
    a query parameter, and Collection information in the request body.

    Args:
        id (UUID):
        organization_id (UUID):
        body (Collection):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
        organization_id=organization_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PostMoveItemidOrganizationIdBody")


@_attrs_define
class PostMoveItemidOrganizationIdBody:
    """
    Attributes:
        array (Union[Unset, list[UUID]]):
    """

    array: Union[Unset, list[UUID]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        array: Union[Unset, list[str]] = UNSET
        if not isinstance(self.array, Unset):
            array = []
            for array_item_data in self.array:
                array_item = str(array_item_data)
                array.append(array_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if array is not UNSET:
            field_dict["array"] = array

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        array = []
        _array = d.pop("array", UNSET)
        for array_item_data in _array or []:
            array_item = UUID(array_item_data)

            array.append(array_item)

        post_move_itemid_organization_id_body = cls(
            array=array,
        )

        post_move_itemid_organization_id_body.additional_properties = d
        return post_move_itemid_organization_id_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response


def _get_kwargs(
    id: UUID,
    *,
    organization_id: UUID,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_organization_id = str(organization_id)
    params["organizationId"] = json_organization_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/object/org-collection/{id}",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 200:
        return None
    if response.status_code == 400:
        return None
    if response.status_code == 404:
        return None
    if response.status_code == 500:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    organization_id: UUID,
) -> Response[Any]:
    """Retrieve a Collection from a specified Organization.

     Retrieve an existing collection from a specified Organization by specifying the unique Collection
    identifier (e.g. `3a84be8d-12e7-4223-98cd-ae0000eabdec`) in the path and an Organization identifier
    as a query parameter .

    Args:
        id (UUID):
        organization_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        organization_id=organization_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    id: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    organization_id: UUID,
) -> Response[Any]:
    """Retrieve a Collection from a specified Organization.

     Retrieve an existing collection from a specified Organization by specifying the unique Collection
    identifier (e.g. `3a84be8d-12e7-4223-98cd-ae0000eabdec`) in the path and an Organization identifier
    as a query parameter .

    Args:
        id (UUID):
        organization_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        organization_id=organization_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.group import Group


T = TypeVar("T", bound="Collection")


@_attrs_define
class Collection:
    """
    Attributes:
        external_id (Union[Unset, str]):
        groups (Union[Unset, list['Group']]):
        name (Union[Unset, str]):
        organization_id (Union[Unset, UUID]):
    """

    external_id: Union[Unset, str] = UNSET
    groups: Union[Unset, list["Group"]] = UNSET
    name: Union[Unset, str] = UNSET
    organization_id: Union[Unset, UUID] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        external_id = self.external_id

        groups: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.groups, Unset):
            groups = []
            for groups_item_data in self.groups:
                groups_item = groups_item_data.to_dict()
                groups.append(groups_item)

        name = self.name

        organization_id: Union[Unset, str] = UNSET
        if not isinstance(self.organization_id, Unset):
            organization_id = str(self.organization_id)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if groups is not UNSET:
            field_dict["groups"] = groups
        if name is not UNSET:
            field_dict["name"] = name
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.group import Group

        d = src_dict.copy()
        external_id = d.pop("externalId", UNSET)

        groups = []
        _groups = d.pop("groups", UNSET)
        for groups_item_data in _groups or []:
            groups_item = Group.from_dict(groups_item_data)

            groups.append(groups_item)

        name = d.pop("name", UNSET)

        _organization_id = d.pop("organizationId", UNSET)
        organization_id: Union[Unset, UUID]
        if isinstance(_organization_id, Unset):
            organization_id = UNSET
        else:
            organization_id = UUID(_organization_id)

        collection = cls(
            external_id=external_id,
            groups=groups,
            name=name,
            organization_id=organization_id,
        )

        collection.additional_properties = d
        return collection

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetObjectTemplateTypeType(str, Enum):
    COLLECTION = "collection"
    FOLDER = "folder"
    ITEM = "item"
    ITEM_CARD = "item.card"
    ITEM_COLLECTIONS = "item-collections"
    ITEM_FIELD = "item.field"
    ITEM_IDENTITY = "item.identity"
    ITEM_LOGIN = "item.login"
    ITEM_LOGIN_URI = "item.login.uri"
    ITEM_SECURENOTE = "item.securenote"
    ORG_COLLECTION = "org-collection"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.uris_match import UrisMatch
from ..types import UNSET, Unset

T = TypeVar("T", bound="Uris")


@_attrs_define
class Uris:
    """
    Attributes:
        match (Union[Unset, UrisMatch]):
        uri (Union[Unset, str]):
    """

    match: Union[Unset, UrisMatch] = UNSET
    uri: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        match: Union[Unset, int] = UNSET
        if not isinstance(self.match, Unset):
            match = self.match.value

        uri = self.uri

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if match is not UNSET:
            field_dict["match"] = match
        if uri is not UNSET:
            field_dict["uri"] = uri

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        _match = d.pop("match", UNSET)
        match: Union[Unset, UrisMatch]
        if isinstance(_match, Unset):
            match = UNSET
        else:
            match = UrisMatch(_match)

        uri = d.pop("uri", UNSET)

        uris = cls(
            match=match,
            uri=uri,
        )

        uris.additional_properties = d
        return uris

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class StatusDataTemplateStatus(str, Enum):
    LOCKED = "locked"
    UNAUTHENTICATED = "unauthenticated"
    UNLOCKED = "unlocked"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ItemIdentity")


@_attrs_define
class ItemIdentity:
    """
    Attributes:
        address1 (Union[Unset, str]):
        address2 (Union[Unset, str]):
        address3 (Union[Unset, str]):
        city (Union[Unset, str]):
        company (Union[Unset, str]):
        country (Union[Unset, str]):
        email (Union[Unset, str]):
        first_name (Union[Unset, str]):
        last_name (Union[Unset, str]):
        license_number (Union[Unset, str]):
        middle_name (Union[Unset, str]):
        passport_number (Union[Unset, str]):
        phone (Union[Unset, str]):
        postal_code (Union[Unset, str]):
        ssn (Union[Unset, str]):
        state (Union[Unset, str]):
        title (Union[Unset, str]):
        username (Union[Unset, str]):
    """

    address1: Union[Unset, str] = UNSET
    address2: Union[Unset, str] = UNSET
    address3: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    company: Union[Unset, str] = UNSET
    country: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    license_number: Union[Unset, str] = UNSET
    middle_name: Union[Unset, str] = UNSET
    passport_number: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET
    postal_code: Union[Unset, str] = UNSET
    ssn: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    username: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        address1 = self.address1

        address2 = self.address2

        address3 = self.address3

        city = self.city

        company = self.company

        country = self.country

        email = self.email

        first_name = self.first_name

        last_name = self.last_name

        license_number = self.license_number

        middle_name = self.middle_name

        passport_number = self.passport_number

        phone = self.phone

        postal_code = self.postal_code

        ssn = self.ssn

        state = self.state

        title = self.title

        username = self.username

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address1 is not UNSET:
            field_dict["address1"] = address1
        if address2 is not UNSET:
            field_dict["address2"] = address2
        if address3 is not UNSET:
            field_dict["address3"] = address3
        if city is not UNSET:
            field_dict["city"] = city
        if company is not UNSET:
            field_dict["company"] = company
        if country is not UNSET:
            field_dict["country"] = country
        if email is not UNSET:
            field_dict["email"] = email
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if license_number is not UNSET:
            field_dict["licenseNumber"] = license_number
        if middle_name is not UNSET:
            field_dict["middleName"] = middle_name
        if passport_number is not UNSET:
            field_dict["passportNumber"] = passport_number
        if phone is not UNSET:
            field_dict["phone"] = phone
        if postal_code is not UNSET:
            field_dict["postalCode"] = postal_code
        if ssn is not UNSET:
            field_dict["ssn"] = ssn
        if state is not UNSET:
            field_dict["state"] = state
        if title is not UNSET:
            field_dict["title"] = title
        if username is not UNSET:
            field_dict["username"] = username

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        address1 = d.pop("address1", UNSET)

        address2 = d.pop("address2", UNSET)

        address3 = d.pop("address3", UNSET)

        city = d.pop("city", UNSET)

        company = d.pop("company", UNSET)

        country = d.pop("country", UNSET)

        email = d.pop("email", UNSET)

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        license_number = d.pop("licenseNumber", UNSET)

        middle_name = d.pop("middleName", UNSET)

        passport_number = d.pop("passportNumber", UNSET)

        phone = d.pop("phone", UNSET)

        postal_code = d.pop("postalCode", UNSET)

        ssn = d.pop("ssn", UNSET)

        state = d.pop("state", UNSET)

        title = d.pop("title", UNSET)

        username = d.pop("username", UNSET)

        item_identity = cls(
            address1=address1,
            address2=address2,
            address3=address3,
            city=city,
            company=company,
            country=country,
            email=email,
            first_name=first_name,
            last_name=last_name,
            license_number=license_number,
            middle_name=middle_name,
            passport_number=passport_number,
            phone=phone,
            postal_code=postal_code,
            ssn=ssn,
            state=state,
            title=title,
            username=username,
        )

        item_identity.additional_properties = d
        return item_identity

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.send_template_type import SendTemplateType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.send_text import SendText


T = TypeVar("T", bound="SendTemplate")


@_attrs_define
class SendTemplate:
    """
    Attributes:
        deletion_date (Union[Unset, datetime.datetime]):
        disabled (Union[Unset, bool]):
        expiration_date (Union[Unset, datetime.datetime]):
        file (Union[Unset, str]):
        hide_email (Union[Unset, bool]):
        max_access_count (Union[Unset, int]):
        name (Union[Unset, str]):
        notes (Union[Unset, str]):
        password (Union[Unset, str]):
        text (Union[Unset, SendText]):
        type_ (Union[Unset, SendTemplateType]):
    """

    deletion_date: Union[Unset, datetime.datetime] = UNSET
    disabled: Union[Unset, bool] = UNSET
    expiration_date: Union[Unset, datetime.datetime] = UNSET
    file: Union[Unset, str] = UNSET
    hide_email: Union[Unset, bool] = UNSET
    max_access_count: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    notes: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    text: Union[Unset, "SendText"] = UNSET
    type_: Union[Unset, SendTemplateType] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        deletion_date: Union[Unset, str] = UNSET
        if not isinstance(self.deletion_date, Unset):
            deletion_date = self.deletion_date.isoformat()

        disabled = self.disabled

        expiration_date: Union[Unset, str] = UNSET
        if not isinstance(self.expiration_date, Unset):
            expiration_date = self.expiration_date.isoformat()

        file = self.file

        hide_email = self.hide_email

        max_access_count = self.max_access_count

        name = self.name

        notes = self.notes

        password = self.password

        text: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.text, Unset):
            text = self.text.to_dict()

        type_: Union[Unset, int] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if deletion_date is not UNSET:
            field_dict["deletionDate"] = deletion_date
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if expiration_date is not UNSET:
            field_dict["expirationDate"] = expiration_date
        if file is not UNSET:
            field_dict["file"] = file
        if hide_email is not UNSET:
            field_dict["hideEmail"] = hide_email
        if max_access_count is not UNSET:
            field_dict["maxAccessCount"] = max_access_count
        if name is not UNSET:
            field_dict["name"] = name
        if notes is not UNSET:
            field_dict["notes"] = notes
        if password is not UNSET:
            field_dict["password"] = password
        if text is not UNSET:
            field_dict["text"] = text
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.send_text import SendText

        d = src_dict.copy()
        _deletion_date = d.pop("deletionDate", UNSET)
        deletion_date: Union[Unset, datetime.datetime]
        if isinstance(_deletion_date, Unset):
            deletion_date = UNSET
        else:
            deletion_date = isoparse(_deletion_date)

        disabled = d.pop("disabled", UNSET)

        _expiration_date = d.pop("expirationDate", UNSET)
        expiration_date: Union[Unset, datetime.datetime]
        if isinstance(_expiration_date, Unset):
            expiration_date = UNSET
        else:
            expiration_date = isoparse(_expiration_date)

        file = d.pop("file", UNSET)

        hide_email = d.pop("hideEmail", UNSET)

        max_access_count = d.pop("maxAccessCount", UNSET)

        name = d.pop("name", UNSET)

        notes = d.pop("notes", UNSET)

        password = d.pop("password", UNSET)

        _text = d.pop("text", UNSET)
        text: Union[Unset, SendText]
        if isinstance(_text, Unset):
            text = UNSET
        else:
            text = SendText.from_dict(_text)

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, SendTemplateType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = SendTemplateType(_type_)

        send_template = cls(
            deletion_date=deletion_date,
            disabled=disabled,
            expiration_date=expiration_date,
            file=file,
            hide_email=hide_email,
            max_access_count=max_access_count,
            name=name,
            notes=notes,
            password=password,
            text=text,
            type_=type_,
        )

        send_template.additional_properties = d
        return send_template

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .collection import Collection
from .field import Field
from .field_type import FieldType
from .folder import Folder
from .get_object_template_type_type import GetObjectTemplateTypeType
from .group import Group
from .item_card import ItemCard
from .item_card_brand import ItemCardBrand
from .item_identity import ItemIdentity
from .item_login import ItemLogin
from .item_secure_note import ItemSecureNote
from .item_secure_note_type import ItemSecureNoteType
from .item_template import ItemTemplate
from .item_template_reprompt import ItemTemplateReprompt
from .item_template_type import ItemTemplateType
from .lockunlock_success import LockunlockSuccess
from .lockunlock_success_data import LockunlockSuccessData
from .post_attachment_body import PostAttachmentBody
from .post_move_itemid_organization_id_body import PostMoveItemidOrganizationIdBody
from .post_unlock_body import PostUnlockBody
from .send_template import SendTemplate
from .send_template_type import SendTemplateType
from .send_text import SendText
from .status import Status
from .status_data import StatusData
from .status_data_object import StatusDataObject
from .status_data_template import StatusDataTemplate
from .status_data_template_status import StatusDataTemplateStatus
from .uris import Uris
from .uris_match import UrisMatch

__all__ = (
    "Collection",
    "Field",
    "FieldType",
    "Folder",
    "GetObjectTemplateTypeType",
    "Group",
    "ItemCard",
    "ItemCardBrand",
    "ItemIdentity",
    "ItemLogin",
    "ItemSecureNote",
    "ItemSecureNoteType",
    "ItemTemplate",
    "ItemTemplateReprompt",
    "ItemTemplateType",
    "LockunlockSuccess",
    "LockunlockSuccessData",
    "PostAttachmentBody",
    "PostMoveItemidOrganizationIdBody",
    "PostUnlockBody",
    "SendTemplate",
    "SendTemplateType",
    "SendText",
    "Status",
    "StatusData",
    "StatusDataObject",
    "StatusDataTemplate",
    "StatusDataTemplateStatus",
    "Uris",
    "UrisMatch",
)



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.item_secure_note_type import ItemSecureNoteType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ItemSecureNote")


@_attrs_define
class ItemSecureNote:
    """
    Attributes:
        type_ (Union[Unset, ItemSecureNoteType]):
    """

    type_: Union[Unset, ItemSecureNoteType] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_: Union[Unset, int] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, ItemSecureNoteType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ItemSecureNoteType(_type_)

        item_secure_note = cls(
            type_=type_,
        )

        item_secure_note.additional_properties = d
        return item_secure_note

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.status_data_object import StatusDataObject
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.status_data_template import StatusDataTemplate


T = TypeVar("T", bound="StatusData")


@_attrs_define
class StatusData:
    """
    Attributes:
        object_ (Union[Unset, StatusDataObject]):
        template (Union[Unset, StatusDataTemplate]):
    """

    object_: Union[Unset, StatusDataObject] = UNSET
    template: Union[Unset, "StatusDataTemplate"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        object_: Union[Unset, str] = UNSET
        if not isinstance(self.object_, Unset):
            object_ = self.object_.value

        template: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.template, Unset):
            template = self.template.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if object_ is not UNSET:
            field_dict["object"] = object_
        if template is not UNSET:
            field_dict["template"] = template

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.status_data_template import StatusDataTemplate

        d = src_dict.copy()
        _object_ = d.pop("object", UNSET)
        object_: Union[Unset, StatusDataObject]
        if isinstance(_object_, Unset):
            object_ = UNSET
        else:
            object_ = StatusDataObject(_object_)

        _template = d.pop("template", UNSET)
        template: Union[Unset, StatusDataTemplate]
        if isinstance(_template, Unset):
            template = UNSET
        else:
            template = StatusDataTemplate.from_dict(_template)

        status_data = cls(
            object_=object_,
            template=template,
        )

        status_data.additional_properties = d
        return status_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.item_card_brand import ItemCardBrand
from ..types import UNSET, Unset

T = TypeVar("T", bound="ItemCard")


@_attrs_define
class ItemCard:
    """
    Attributes:
        brand (Union[Unset, ItemCardBrand]):
        cardholder_name (Union[Unset, str]):
        code (Union[Unset, str]):
        exp_month (Union[Unset, str]):
        exp_year (Union[Unset, str]):
        number (Union[Unset, str]):
    """

    brand: Union[Unset, ItemCardBrand] = UNSET
    cardholder_name: Union[Unset, str] = UNSET
    code: Union[Unset, str] = UNSET
    exp_month: Union[Unset, str] = UNSET
    exp_year: Union[Unset, str] = UNSET
    number: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        brand: Union[Unset, str] = UNSET
        if not isinstance(self.brand, Unset):
            brand = self.brand.value

        cardholder_name = self.cardholder_name

        code = self.code

        exp_month = self.exp_month

        exp_year = self.exp_year

        number = self.number

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if brand is not UNSET:
            field_dict["brand"] = brand
        if cardholder_name is not UNSET:
            field_dict["cardholderName"] = cardholder_name
        if code is not UNSET:
            field_dict["code"] = code
        if exp_month is not UNSET:
            field_dict["expMonth"] = exp_month
        if exp_year is not UNSET:
            field_dict["expYear"] = exp_year
        if number is not UNSET:
            field_dict["number"] = number

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        _brand = d.pop("brand", UNSET)
        brand: Union[Unset, ItemCardBrand]
        if isinstance(_brand, Unset):
            brand = UNSET
        else:
            brand = ItemCardBrand(_brand)

        cardholder_name = d.pop("cardholderName", UNSET)

        code = d.pop("code", UNSET)

        exp_month = d.pop("expMonth", UNSET)

        exp_year = d.pop("expYear", UNSET)

        number = d.pop("number", UNSET)

        item_card = cls(
            brand=brand,
            cardholder_name=cardholder_name,
            code=code,
            exp_month=exp_month,
            exp_year=exp_year,
            number=number,
        )

        item_card.additional_properties = d
        return item_card

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    organization_id: Union[Unset, UUID] = UNSET,
    collection_id: Union[Unset, UUID] = UNSET,
    folderid: Union[Unset, UUID] = UNSET,
    url_query: Union[Unset, str] = UNSET,
    trash: Union[Unset, bool] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_organization_id: Union[Unset, str] = UNSET
    if not isinstance(organization_id, Unset):
        json_organization_id = str(organization_id)
    params["organizationId"] = json_organization_id

    json_collection_id: Union[Unset, str] = UNSET
    if not isinstance(collection_id, Unset):
        json_collection_id = str(collection_id)
    params["collectionId"] = json_collection_id

    json_folderid: Union[Unset, str] = UNSET
    if not isinstance(folderid, Unset):
        json_folderid = str(folderid)
    params["folderid"] = json_folderid

    params["url"] = url_query

    params["trash"] = trash

    params["search"] = search

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/list/object/items",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 200:
        return None
    if response.status_code == 400:
        return None
    if response.status_code == 404:
        return None
    if response.status_code == 500:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    organization_id: Union[Unset, UUID] = UNSET,
    collection_id: Union[Unset, UUID] = UNSET,
    folderid: Union[Unset, UUID] = UNSET,
    url_query: Union[Unset, str] = UNSET,
    trash: Union[Unset, bool] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Retrieve a list of items in your vault.

     Retrieve a list of existing items in your vault. By default, this will return a list of all existing
    items in your vault, however you can specify filters or search terms as query parameters to narrow
    list results.<br><br>Using multiple filters will perform a logical `OR` operation. Using filters
    **and** search terms will perform a logical `AND` operation.

    Args:
        organization_id (Union[Unset, UUID]):
        collection_id (Union[Unset, UUID]):
        folderid (Union[Unset, UUID]):
        url_query (Union[Unset, str]):
        trash (Union[Unset, bool]):
        search (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        organization_id=organization_id,
        collection_id=collection_id,
        folderid=folderid,
        url_query=url_query,
        trash=trash,
        search=search,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    organization_id: Union[Unset, UUID] = UNSET,
    collection_id: Union[Unset, UUID] = UNSET,
    folderid: Union[Unset, UUID] = UNSET,
    url_query: Union[Unset, str] = UNSET,
    trash: Union[Unset, bool] = UNSET,
    search: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Retrieve a list of items in your vault.

     Retrieve a list of existing items in your vault. By default, this will return a list of all existing
    items in your vault, however you can specify filters or search terms as query parameters to narrow
    list results.<br><br>Using multiple filters will perform a logical `OR` operation. Using filters
    **and** search terms will perform a logical `AND` operation.

    Args:
        organization_id (Union[Unset, UUID]):
        collection_id (Union[Unset, UUID]):
        folderid (Union[Unset, UUID]):
        url_query (Union[Unset, str]):
        trash (Union[Unset, bool]):
        search (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        organization_id=organization_id,
        collection_id=collection_id,
        folderid=folderid,
        url_query=url_query,
        trash=trash,
        search=search,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    length: Union[Unset, int] = UNSET,
    uppercase: Union[Unset, bool] = UNSET,
    lowercase: Union[Unset, bool] = UNSET,
    number: Union[Unset, bool] = UNSET,
    special: Union[Unset, bool] = UNSET,
    passphrase: Union[Unset, bool] = UNSET,
    words: Union[Unset, int] = UNSET,
    separator: Union[Unset, str] = UNSET,
    capitalize: Union[Unset, bool] = UNSET,
    include_number: Union[Unset, bool] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["length"] = length

    params["uppercase"] = uppercase

    params["lowercase"] = lowercase

    params["number"] = number

    params["special"] = special

    params["passphrase"] = passphrase

    params["words"] = words

    params["separator"] = separator

    params["capitalize"] = capitalize

    params["includeNumber"] = include_number

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/generate",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == 200:
        return None
    if response.status_code == 400:
        return None
    if response.status_code == 404:
        return None
    if response.status_code == 500:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    length: Union[Unset, int] = UNSET,
    uppercase: Union[Unset, bool] = UNSET,
    lowercase: Union[Unset, bool] = UNSET,
    number: Union[Unset, bool] = UNSET,
    special: Union[Unset, bool] = UNSET,
    passphrase: Union[Unset, bool] = UNSET,
    words: Union[Unset, int] = UNSET,
    separator: Union[Unset, str] = UNSET,
    capitalize: Union[Unset, bool] = UNSET,
    include_number: Union[Unset, bool] = UNSET,
) -> Response[Any]:
    """Generate a password or passphrase.

     Generate a password or passphrase. By default, `/generate` will generate a 14-character password
    with uppercase characters, lowercase characters, and numbers.

    Args:
        length (Union[Unset, int]):
        uppercase (Union[Unset, bool]):
        lowercase (Union[Unset, bool]):
        number (Union[Unset, bool]):
        special (Union[Unset, bool]):
        passphrase (Union[Unset, bool]):
        words (Union[Unset, int]):
        separator (Union[Unset, str]):
        capitalize (Union[Unset, bool]):
        include_number (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        length=length,
        uppercase=uppercase,
        lowercase=lowercase,
        number=number,
        special=special,
        passphrase=passphrase,
        words=words,
        separator=separator,
        capitalize=capitalize,
        include_number=include_number,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    length: Union[Unset, int] = UNSET,
    uppercase: Union[Unset, bool] = UNSET,
    lowercase: Union[Unset, bool] = UNSET,
    number: Union[Unset, bool] = UNSET,
    special: Union[Unset, bool] = UNSET,
    passphrase: Union[Unset, bool] = UNSET,
    words: Union[Unset, int] = UNSET,
    separator: Union[Unset, str] = UNSET,
    capitalize: Union[Unset, bool] = UNSET,
    include_number: Union[Unset, bool] = UNSET,
) -> Response[Any]:
    """Generate a password or passphrase.

     Generate a password or passphrase. By default, `/generate` will generate a 14-character password
    with uppercase characters, lowercase characters, and numbers.

    Args:
        length (Union[Unset, int]):
        uppercase (Union[Unset, bool]):
        lowercase (Union[Unset, bool]):
        number (Union[Unset, bool]):
        special (Union[Unset, bool]):
        passphrase (Union[Unset, bool]):
        words (Union[Unset, int]):
        separator (Union[Unset, str]):
        capitalize (Union[Unset, bool]):
        include_number (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        length=length,
        uppercase=uppercase,
        lowercase=lowercase,
        number=number,
        special=special,
        passphrase=passphrase,
        words=words,
        separator=separator,
        capitalize=capitalize,
        include_number=include_number,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)
